#!/bin/sh

echo "+++++ Checking current user rights..."

if [ "$(id -u root)" = "$(id -u)" ] ; then
    :
else
    echo "ERROR: The current user id is not root (`id|awk '{print $1}'`)."
    exit 1
fi
echo


if [ "X$PVSW_ROOT" = "X" ] ; then
    PVSW_ROOT=/usr/local/psql
fi

# Determine if system libstdc meets Zen version requirements and remove
# bundled version if it does.
if [ $(uname -s) != "Darwin" ]; then
    echo "+++++ Checking system libstdc++ version support..."
    # Verify readelf is available.
    READELF=`which readelf`
    if [ ! -f "$READELF" ]; then
        echo "WARNING: Unable to determine system libstdc++ version support (missing readelf command)."
        echo "The libstdc++ libraries bundled with the Zen install will be used, which may prevent the"
        echo "opening of documentation from within ZenCC and possibly other issues running Zen utilities."
    else
        ZEN_REQUIRED_LIBSTDC_VER=3.4.21
        ZEN_FILE32_TO_CHECK=$PVSW_ROOT/bin/psregsvr
        ZEN_FILE64_TO_CHECK=$PVSW_ROOT/bin/psregsvr64

        # First rename the bundled libstdc++ libs in case $PVSW_ROOT is
        # already in the path so the ldd check below will locate the system
        # library instead of the Zen-provided library (or not find it at all).
        if [ -f $PVSW_ROOT/lib/libstdc++.so.6 ]; then
            mv $PVSW_ROOT/lib/libstdc++.so.6 $PVSW_ROOT/lib/libstdc++.so.6.zen
        fi
        if [ -f $PVSW_ROOT/lib64/libstdc++.so.6 ]; then
            mv $PVSW_ROOT/lib64/libstdc++.so.6 $PVSW_ROOT/lib64/libstdc++.so.6.zen
        fi

        # Now locate 32-bit system library path from file dependencies for $ZEN_FILE32_TO_CHECK.
        if [ -f $ZEN_FILE32_TO_CHECK ]; then
            echo "Obtaining libstdc++ dependencies from 32-bit $ZEN_FILE32_TO_CHECK"
            SYS_INSTALLED_LIBSTDC32_PATH=`ldd $ZEN_FILE32_TO_CHECK | grep libstdc++.so.6 | cut -d " " -f 3`
            if [ -f "$SYS_INSTALLED_LIBSTDC32_PATH" ]; then
                echo "Checking for required version info in $SYS_INSTALLED_LIBSTDC32_PATH"
                FOUND_ZEN_LIBSTDC32_DEP=`readelf -V $SYS_INSTALLED_LIBSTDC32_PATH | grep GLIBCXX_$ZEN_REQUIRED_LIBSTDC_VER`
                if [ "X$FOUND_ZEN_LIBSTDC32_DEP" = "X" ]; then
                    echo "Required 32-bit version support not found, Zen will use 32-bit libstdc++.so.6 from install."
                    # Revert previous rename of libraries bundled with Zen install.
                    if [ -f $PVSW_ROOT/lib/libstdc++.so.6.zen ]; then
                        # Revert previous rename of libraries bundled with Zen install.
                        mv $PVSW_ROOT/lib/libstdc++.so.6.zen $PVSW_ROOT/lib/libstdc++.so.6
                    fi
                else
                    echo "Required 32-bit version support found, Zen will use $SYS_INSTALLED_LIBSTDC32_PATH"
                    # Must also remove previously renamed file or it will get a simlink created
                    # for it later in this script.
                    if [ -f $PVSW_ROOT/lib/libstdc++.so.6.zen ]; then
                        rm -f $PVSW_ROOT/lib/libstdc++.so.6.zen
                    fi
                fi
            else
                echo "Required 32-bit version support not found, Zen will use 32-bit libstdc++.so.6 from install."
                # Revert previous rename of libraries bundled with Zen install.
                if [ -f $PVSW_ROOT/lib/libstdc++.so.6.zen ]; then
                    # Revert previous rename of libraries bundled with Zen install.
                    mv $PVSW_ROOT/lib/libstdc++.so.6.zen $PVSW_ROOT/lib/libstdc++.so.6
                fi
            fi
        else
            echo "The 32-bit $ZEN_FILE32_TO_CHECK is not present in the Zen install package, skipping 32-bit libstdc++ dependency check."
        fi

        # Now locate 64-bit system library path from file dependencies for $ZEN_FILE64_TO_CHECK.
        if [ -f $ZEN_FILE64_TO_CHECK ]; then
            echo "Obtaining libstdc++ dependencies from 64-bit $ZEN_FILE64_TO_CHECK"
            SYS_INSTALLED_LIBSTDC64_PATH=`ldd $ZEN_FILE64_TO_CHECK | grep libstdc++.so.6 | cut -d " " -f 3`
            if [ -f "$SYS_INSTALLED_LIBSTDC64_PATH" ]; then
                echo "Checking for required version info in $SYS_INSTALLED_LIBSTDC64_PATH"
                FOUND_ZEN_LIBSTDC64_DEP=`readelf -V $SYS_INSTALLED_LIBSTDC64_PATH | grep GLIBCXX_$ZEN_REQUIRED_LIBSTDC_VER`
                if [ "X$FOUND_ZEN_LIBSTDC64_DEP" = "X" ]; then
                    echo "Required 64-bit version support not found, Zen will use 64-bit libstdc++.so.6 from install."
                    # Revert previous rename of libraries bundled with Zen install.
                    if [ -f $PVSW_ROOT/lib64/libstdc++.so.6.zen ]; then
                        # Revert previous rename of libraries bundled with Zen install.
                        mv $PVSW_ROOT/lib64/libstdc++.so.6.zen $PVSW_ROOT/lib64/libstdc++.so.6
                    fi
                else
                    echo "Required 64-bit version support found, Zen will use $SYS_INSTALLED_LIBSTDC64_PATH"
                    # Must also remove previously renamed file or it will get a simlink created
                    # for it later in this script.
                    if [ -f $PVSW_ROOT/lib64/libstdc++.so.6.zen ]; then
                        rm -f $PVSW_ROOT/lib64/libstdc++.so.6.zen
                    fi
                fi
            else
                echo "Required 64-bit version support not found, Zen will use 64-bit libstdc++.so.6 from install."
                # Revert previous rename of libraries bundled with Zen install.
                if [ -f $PVSW_ROOT/lib64/libstdc++.so.6.zen ]; then
                    # Revert previous rename of libraries bundled with Zen install.
                    mv $PVSW_ROOT/lib64/libstdc++.so.6.zen $PVSW_ROOT/lib64/libstdc++.so.6
                fi
            fi
        else
            echo "The 64-bit $ZEN_FILE64_TO_CHECK is not present in the Zen install, skipping 64-bit dependency check."
        fi
    fi
fi
echo

PATH=$PVSW_ROOT/bin:$PATH

if [ $(uname -s) = "Darwin" ]
then
    DYLD_LIBRARY_PATH=$PVSW_ROOT/bin:$PVSW_ROOT/lib64:$DYLD_LIBRARY_PATH
    export DYLD_LIBRARY_PATH
    SO_EXTENSION="dylib"
    SU_ARGS="-l"
else
    LD_LIBRARY_PATH=$PVSW_ROOT/bin:$PVSW_ROOT/lib:$PVSW_ROOT/lib64:$LD_LIBRARY_PATH
    export LD_LIBRARY_PATH
    SO_EXTENSION="so"
    SU_ARGS=
fi

export PVSW_ROOT
if [ -f $PVSW_ROOT/etc/postinstall.cfg ] ; then
    . $PVSW_ROOT/etc/postinstall.cfg
fi
SU="su -"
BASH_ENV=
USERNAME=psql
GROUPNAME=pvsw
ADMGROUPNAME=pvsw-adm
PS=/bin/ps
PSPARAMS=ax
SEM=sem
MEM=shm
ECHO=/bin/echo
GRPFLAG="-g 5000"
ADMGRPFLAG="-g 5001"
USERSHL=/bin/bash
USERSHLPROFILE1=.bash_profile
USERSHLPROFILE2=.bashrc
SECDIR=/lib/security
if [ ! -d $SECDIR ] ; then
    if [ $(uname -s) = "Darwin" ]
    then
        SECDIR=/usr/lib/pam
    else
        SECDIR=/usr/lib/security
    fi
fi
if [ ! -d $SECDIR ] ; then
    SECDIR=
fi
SORT=/bin/sort
if [ ! -x $SORT ] ; then
    SORT=/usr/bin/sort
fi
LDCONFIG=/sbin/ldconfig
if [ ! -x $LDCONFIG ] ; then
    LDCONFIG=/usr/sbin/ldconfig
fi
PWDBLIB=pam_pwdb.${SO_EXTENSION}
PWDBLIBPARAM="shadow nullok"
USERSHLPROFILE3=.cshrc
SELINUX_ENABLED=/usr/sbin/selinuxenabled


echo "+++++ Setting up $GROUPNAME group and admgroup..."

if [ $(uname -s) = "Darwin" ]
then
    GPVSWEXISTS=`dscl . -list /Groups PrimaryGroupID | awk '($1 == "'"$GROUPNAME"'")'`
else
    GPVSWEXISTS=`grep "^$GROUPNAME:" /etc/group`
fi

if [ "X$GPVSWEXISTS" != "X" ] ; then
    echo "An existing group $GROUPNAME was found."
else
    echo "Creating group $GROUPNAME..."

    if [ $(uname -s) = "Darwin" ]
    then
        dscl . -delete /Groups/"$GROUPNAME" 2>/dev/null
        groupid="$(($(dscl . -list /Groups PrimaryGroupID | awk '{print $2}' | sort -n | tail -n 1) + 1))"
        dscl . -create /Groups/"$GROUPNAME" PrimaryGroupID "$groupid"
    else
        /usr/sbin/groupadd $GRPFLAG $GROUPNAME
    fi
fi

if [ $(uname -s) = "Darwin" ]
then
    GPVSWEXISTS=`dscl . -list /Groups PrimaryGroupID | awk '($1 == "'"$ADMGROUPNAME"'")'`
else
    GPVSWEXISTS=`grep "^$ADMGROUPNAME:" /etc/group`
fi

if [ "X$GPVSWEXISTS" != "X" ] ; then
    echo "An existing group $ADMGROUPNAME was found."
else
    echo "Creating group $ADMGROUPNAME..."

    if [ $(uname -s) = "Darwin" ]
    then
        dscl . -delete /Groups/"$ADMGROUPNAME" 2>/dev/null
        dscl . -create /Groups/"$ADMGROUPNAME" PrimaryGroupID "$(($(dscl . -list /Groups PrimaryGroupID | awk '{print $2}' | sort -n | tail -n 1) + 1))"
    else
        /usr/sbin/groupadd $ADMGRPFLAG $ADMGROUPNAME
    fi
fi
echo


# Creating the root and bin directories if they don't exist
if [ ! -d $PVSW_ROOT ] ; then
    mkdir $PVSW_ROOT
fi
if [ ! -d $PVSW_ROOT/bin ] ; then
    mkdir $PVSW_ROOT/bin
fi

echo "+++++ Setting up user $USERNAME..."

if [ $(uname -s) = "Darwin" ]
then
    UPVSWEXISTS=`dscl . -list /Users UniqueID | awk '($1 == "'"$USERNAME"'")'`
else
    UPVSWEXISTS=`grep "^$USERNAME:" /etc/passwd`
fi

if [ "X$UPVSWEXISTS" != "X" ] ; then
    echo "An existing user $USERNAME was found."
    
    # check if there are any problems with the existing user
    if [ $(uname -s) = "Darwin" ]
    then
        CPVSWSHELL=`dscl . -list /Users UserShell | awk '($1 == "'"$USERNAME"'")' | grep $USERSHL`
    else
        CPVSWSHELL=`echo $UPVSWEXISTS|grep $USERSHL`
    fi

    if [ "X$CPVSWSHELL" = "X" ] ; then
        echo "ERROR: Existing user $USERNAME has shell another than $USERSHL."
        exit 1 ;
    fi
else
    echo "Creating user $USERNAME..."

    if [ $(uname -s) = "Darwin" ]
    then
        HOMEDIR=/Users/"$USERNAME"
        dscl . -create "$HOMEDIR" UniqueID "$(($(dscl . -list /Users UniqueID | awk '{print $2}' | sort -n | tail -n 1) + 1))"
        dscl . -create "$HOMEDIR" RealName "PSQL"
        dscl . -create "$HOMEDIR" UserShell "$USERSHL"
        dscl . -create "$HOMEDIR" PrimaryGroupID "$(dscl . -list /Groups PrimaryGroupID | awk '($1 == "'"$GROUPNAME"'") { print $2 }')"
        dscl . -create "$HOMEDIR" NFSHomeDirectory "$HOMEDIR"
        defaults write /Library/Preferences/com.apple.loginwindow HiddenUsersList -array-add "$USERNAME"
        mkdir -p "$HOMEDIR"
        chown psql:pvsw "$HOMEDIR"
        chmod 755 "$HOMEDIR"
        unset HOMEDIR
    else
        /usr/sbin/useradd -c "PSQL" -m -g $GROUPNAME -s $USERSHL $USERNAME
    fi

    if [ "$?" != "0" ]; then
        echo "ERROR: The user $USERNAME could not be created."
        exit 1;
    fi
    sleep 2

    # Removed loop
    TEMPFILE=/tmp/psql.tmp
    touch $TEMPFILE

    chown $USERNAME $TEMPFILE

    if [ "$?" != "0" ]; then
        echo "ERROR: The user $USERNAME could not be created."
        exit 1;
    fi
    rm -f $TEMPFILE
fi

echo "Configuring user $USERNAME environment..."
if [ $(uname -s) = "Darwin" ]
then
    HOMEDIR=/Users/psql
else
    # If there's an xhost command in our path.
    if which xhost >/dev/null 2>&1
    then
        if [ -f $PVSW_ROOT/bin/pcc ] ; then
            xhost local:$USERNAME
        fi
    fi

    HOMEDIR=`grep "^$USERNAME:" /etc/passwd | cut -d: -f6`
fi

PROFILE1=$HOMEDIR/$USERSHLPROFILE1
PROFILE2=$HOMEDIR/$USERSHLPROFILE2
PROFILE3=$HOMEDIR/$USERSHLPROFILE3

if [ $(uname -s) = "Darwin" ]
then
    if [ "$LANG" = "" ]
    then
        LANG="PSQL_LANG"
    fi

    echo "User \"$USERNAME\" LANG environment variable will be set to \"$LANG\"."
fi

cat > $PROFILE1 << EOPF
umask 022
export PVSW_ROOT=$PVSW_ROOT
export PATH=\$PVSW_ROOT/bin:/bin:/usr/bin

if [ \$(uname -s) = "Darwin" ]
then
    export DYLD_LIBRARY_PATH=\$PVSW_ROOT/lib64:\$PVSW_ROOT/bin:/usr/lib
    export LANG=$LANG
else
    export LD_LIBRARY_PATH=\$PVSW_ROOT/lib:\$PVSW_ROOT/lib64:\$PVSW_ROOT/bin:/usr/lib
fi

export MANPATH=\$PVSW_ROOT/man:\$MANPATH
export BREQ=\$PVSW_ROOT/lib
export LD_BIND_NOW=1

EOPF

cp -f $PROFILE1 $PROFILE2
if [ ! -f $PROFILE3 ] ; then
cat > $PROFILE3 <<EOPF2
setenv PVSW_ROOT $PVSW_ROOT

if [ \$(uname -s) = "Darwin" ]
then
    setenv DYLD_LIBRARY_PATH \$PVSW_ROOT/lib64:\$DYLD_LIBRARY_PATH
    setenv LANG $LANG
else
    setenv LD_LIBRARY_PATH \$PVSW_ROOT/lib:\$PVSW_ROOT/lib64:\$LD_LIBRARY_PATH
fi

setenv MANPATH \$PVSW_ROOT/man:\$MANPATH
set path = (\$PVSW_ROOT/bin $path)
setenv BREQ \$PVSW_ROOT/lib
setenv LD_BIND_NOW 1

EOPF2
fi
echo


echo "+++++ Setting up symbolic links for libraries..."

# These used to be in the RPM spec file, but we moved them
# here for tarball installs.
rm -f $PVSW_ROOT/lib/libbtrvif.${SO_EXTENSION}

if [ ! $(uname -s) = "Darwin" ]
then
    # Use ldconfig to create links to major versions
    # selinux: ldconfig runs in its own domain and may not have
    # permissions to write to tty.
    $LDCONFIG -n $PVSW_ROOT/lib 2>&1 | cat
    if [ -d $PVSW_ROOT/lib64 ]; then
        $LDCONFIG -n $PVSW_ROOT/lib64 2>&1 | cat
    fi

    MIF_FILE=`ls -1 $PVSW_ROOT/lib/libpsqlmif.${SO_EXTENSION}.?? 2> /dev/null | tail -1`
    if [ "x$MIF_FILE" = "x" ]; then
        MIF_FILE=`ls -1 $PVSW_ROOT/lib64/libpsqlmif.${SO_EXTENSION}.?? 2> /dev/null | tail -1`
    fi
    PSCL_FILE=`ls -1 $PVSW_ROOT/lib/libpscl.${SO_EXTENSION}.? 2> /dev/null | tail -1`
    if [ "x$PSCL_FILE" = "x" ]; then
        PSCL_FILE=`ls -1 $PVSW_ROOT/lib64/libpscl.${SO_EXTENSION}.? 2> /dev/null | tail -1`
    fi
    PSQL_VER=`echo $MIF_FILE | sed -e 's/.*[.]\(.*\)/\1/'`
    PS_VER=$PSQL_VER

    dso_symlink () {
        dso=$1
        ver=$2
        if [ -f $PVSW_ROOT/lib/$dso.$ver ]; then
            (cd $PVSW_ROOT/lib; ln -sf $dso.$ver $dso )
        fi
        if [ -f $PVSW_ROOT/lib64/$dso.$ver ]; then
            (cd $PVSW_ROOT/lib64; ln -sf $dso.$ver $dso )
        fi
    }
else
    MIF_FILE=`ls -1 $PVSW_ROOT/lib64/libpsqlmif.${SO_EXTENSION}.[0-9]* 2> /dev/null | tail -1`
    if [ "x$MIF_FILE" = "x" ]; then
        MIF_FILE=`ls -1 $PVSW_ROOT/lib64/libpsqlmif.${SO_EXTENSION}.[0-9]* 2> /dev/null | tail -1`
    fi
    PSQL_VER=`echo $MIF_FILE | sed -e 's/^.*\.'"${SO_EXTENSION}"'\.\([0-9]*\)\..*$/\1/'`
    PS_VER=$PSQL_VER

    dso_symlink () {
      dso=$1
      ver=$2
      if [ -f $PVSW_ROOT/lib64/$dso.$ver.[0-9]*.* ]; then
          (cd $PVSW_ROOT/lib64; ln -sf $dso.$ver.[0-9]*.* $dso.$ver ; ln -sf $dso.$ver $dso )
      fi
    }

    dso_symlink libbdulb.${SO_EXTENSION} $PSQL_VER
    dso_symlink libbdumsgrb.${SO_EXTENSION} $PSQL_VER
    dso_symlink libclientrb.${SO_EXTENSION} $PSQL_VER
    dso_symlink libcobolschemaexecmsgrb.${SO_EXTENSION} $PSQL_VER
    dso_symlink libdbcsipxy.${SO_EXTENSION} $PSQL_VER
    dso_symlink libdcm100.${SO_EXTENSION} $PSQL_VER
    dso_symlink libenginelm.${SO_EXTENSION} $PSQL_VER
    dso_symlink libexp010.${SO_EXTENSION} $PSQL_VER
    dso_symlink liblegacylm.${SO_EXTENSION} $PSQL_VER
    dso_symlink liblicmgrrb.${SO_EXTENSION} $PSQL_VER
    dso_symlink libmkc3.${SO_EXTENSION} $PSQL_VER
    dso_symlink libmkderb.${SO_EXTENSION} $PSQL_VER
    dso_symlink libodbccr.${SO_EXTENSION} 1
    dso_symlink libpceurop.${SO_EXTENSION} $PS_VER
    dso_symlink libpctlgrb.${SO_EXTENSION} $PSQL_VER
    dso_symlink libpscl.${SO_EXTENSION} $PS_VER
    dso_symlink libpscore.${SO_EXTENSION} $PS_VER
    dso_symlink libpscp932.${SO_EXTENSION} $PS_VER
    dso_symlink libpsdom.${SO_EXTENSION} $PS_VER
    dso_symlink libpseucjp.${SO_EXTENSION} $PS_VER
    dso_symlink libpssax.${SO_EXTENSION} $PS_VER
    dso_symlink libpsutilrb.${SO_EXTENSION} $PS_VER
    dso_symlink libpvmsgrb.${SO_EXTENSION} $PSQL_VER
    dso_symlink libsrderb.${SO_EXTENSION} $PSQL_VER
    dso_symlink libupiapirb.${SO_EXTENSION} $PSQL_VER
    dso_symlink libxlate.${SO_EXTENSION} $PSQL_VER
fi

dso_symlink libodbc.${SO_EXTENSION} 1
dso_symlink libpsqldti.${SO_EXTENSION} $PSQL_VER
dso_symlink libclientlm.${SO_EXTENSION} $PSQL_VER

dso_symlink libels.${SO_EXTENSION} '*.*.*.*'

dso_symlink libbtrieveC.${SO_EXTENSION} $PSQL_VER
dso_symlink libbtrieveCpp.${SO_EXTENSION} $PSQL_VER
dso_symlink libpsqlmif.${SO_EXTENSION} $PSQL_VER
dso_symlink libpsqlnsl.${SO_EXTENSION} $PSQL_VER
dso_symlink libpsqlcsm.${SO_EXTENSION} $PSQL_VER
dso_symlink libpsqlcsp.${SO_EXTENSION} $PSQL_VER
dso_symlink libodbcci.${SO_EXTENSION} $PSQL_VER
dso_symlink libcsi100.${SO_EXTENSION} $PSQL_VER

echo

cd $PVSW_ROOT/bin 
if [ -f $PVSW_ROOT/bin/psregedit64 ] ; then
    ln -sf psregedit64 psregedit
fi
if [ -f $PVSW_ROOT/bin/clilcadm64 ] ; then
    ln -sf clilcadm64 clilcadm
fi
if [ -f $PVSW_ROOT/bin/licgetauth64 ] ; then
    ln -sf licgetauth64 licgetauth
fi
if [ -f $PVSW_ROOT/bin/isql64 ] ; then
    ln -sf isql64 isql
fi
cd $PVSW_ROOT

if [ -x $SELINUX_ENABLED ] && $SELINUX_ENABLED; then
    /sbin/restorecon -R $PVSW_ROOT
    if [ -d "$PVSW_ROOT"/lib64 ]
    then
        /usr/bin/chcon -t textrel_shlib_t "$PVSW_ROOT"/lib64/*.so "$PVSW_ROOT"/lib64/*.so.*
    fi
fi

echo "+++++ Making user $USERNAME owner of all files in $PVSW_ROOT..."

chown -Rf $USERNAME:$GROUPNAME $PVSW_ROOT
chmod g+w $PVSW_ROOT/etc
# install-related scripts remain owned by root so psql cannot uninstall (which fails)
if [ $(uname -s) = "Darwin" ]
then
    chown root:wheel $PVSW_ROOT/etc/*install.sh
else
    chown root:root $PVSW_ROOT/etc/*install.sh
fi
chmod 770 $PVSW_ROOT/etc/*install.sh
echo


echo "+++++ Registering the PCOM libraries..."

# Sym links must already exist
dso_register () {
    dso=$1
    ver=$2
    if [ -f $PVSW_ROOT/lib/$dso.$ver ]; then
        $PVSW_ROOT/bin/psregsvr $PVSW_ROOT/lib/$dso.$ver
        if [ $? -ne 0 ]; then
            echo ERROR: PCOM registration failed for $dso.$ver.
            exit 1;
        fi
    fi
    if [ -f $PVSW_ROOT/lib64/$dso.$ver ]; then
        $PVSW_ROOT/bin/psregsvr64 $PVSW_ROOT/lib64/$dso.$ver
        if [ $? -ne 0 ]; then
            echo ERROR: PCOM registration failed for $dso.$ver.
            exit 1;
        fi
    fi
}
dso_register libpctlgrb.${SO_EXTENSION} $PSQL_VER
dso_register libcobolschemaexecmsgrb.${SO_EXTENSION} $PSQL_VER
dso_register libpceurop.${SO_EXTENSION} $PS_VER
dso_register libpssax.${SO_EXTENSION} $PS_VER
dso_register libpsdom.${SO_EXTENSION} $PS_VER
dso_register libpsutilrb.${SO_EXTENSION} $PS_VER
dso_register libclientrb.${SO_EXTENSION} $PSQL_VER
dso_register libupiapirb.${SO_EXTENSION} $PSQL_VER
dso_register libcsi100.${SO_EXTENSION} $PSQL_VER
dso_register libmkc3.${SO_EXTENSION} $PSQL_VER
dso_register libpvmsgrb.${SO_EXTENSION} $PSQL_VER
dso_register libpsqlcsm.${SO_EXTENSION} $PSQL_VER
dso_register libpsqlcsp.${SO_EXTENSION} $PSQL_VER
dso_register libdbcsipxy.${SO_EXTENSION} $PSQL_VER
dso_register libpsqlmpm.${SO_EXTENSION} $PSQL_VER
dso_register libpscp932.${SO_EXTENSION} $PS_VER
dso_register libsrderb.${SO_EXTENSION} $PSQL_VER

# Reset owner to psql.pvsw
PS_REGISTRY=$PVSW_ROOT/etc/.PSRegistry
PS_SEM_IPC_SEM_ID="/tmp/PS_sem_*_IPC_SEM.id"
PS_SHM_IPC_SHARED_MEM_ID="/tmp/PS_shm_*_IPC_SHARED_MEM.id"

# If PS_REGISTRY exists.
if [ -e $PS_REGISTRY ] ; then
    chown -Rf $USERNAME:$GROUPNAME $PS_REGISTRY

	# If there are any PS_SEM_IPC_SEM_ID files.
	if ls $PS_SEM_IPC_SEM_ID > /dev/null 2>&1 ; then
		chown -f $USERNAME:$GROUPNAME $PS_SEM_IPC_SEM_ID
	fi

	# If there are any PS_SHM_IPC_SHARED_MEM_ID files.
	if ls $PS_SHM_IPC_SHARED_MEM_ID > /dev/null 2>&1 ; then
		chown -f $USERNAME:$GROUPNAME $PS_SHM_IPC_SHARED_MEM_ID
	fi

    # Add read only for everybody and full control for user and group.
    chmod -R a-rwx  $PS_REGISTRY
    chmod -R a+rx   $PS_REGISTRY
    chmod -R ug+rwx $PS_REGISTRY
fi


################# Creating the DSN ini file ######################
ODBCINI=$PVSW_ROOT/etc/odbc.ini
if [ ! -f $ODBCINI ] ; then
    touch $ODBCINI
    chown psql:pvsw $ODBCINI
else
    echo "An existing DSN $DSNNAME was found."
fi
chmod a-wrx $ODBCINI
chmod a+r $ODBCINI
chmod ug+w $ODBCINI

if [ -f $PVSW_ROOT/bin/dsnadd ] ; then
    chmod a+x $PVSW_ROOT/bin/dsnadd
fi

# Populate the InstallInfo registry settings.
MIF_FILE=`ls -1 $PVSW_ROOT/lib/libpsqlmif.${SO_EXTENSION}.*.*.* 2> /dev/null | tail -1`
if [ "x$MIF_FILE" = "x" ]; then
    MIF_FILE=`ls -1 $PVSW_ROOT/lib64/libpsqlmif.${SO_EXTENSION}.*.*.* 2> /dev/null | tail -1`
fi
PRODUCT_VER=`egrep 'ProductVer=?' $PVSW_ROOT/etc/postinstall.cfg | cut -d= -f2`
VERSION_LEVEL=`egrep 'VersionLevel=?' $PVSW_ROOT/etc/postinstall.cfg | cut -d= -f2`

PSREGEDIT=psregedit
if [ -f $PVSW_ROOT/bin/psregedit64 ] ; then
    PSREGEDIT=psregedit64
fi
$PSREGEDIT -set -key "PS_HKEY_CONFIG/SOFTWARE/Pervasive Software/Products/PSQL Client/InstallInfo" -value VersionLevel $VERSION_LEVEL
$PSREGEDIT -set -key "PS_HKEY_CONFIG/SOFTWARE/Pervasive Software/Products/PSQL Client/InstallInfo" -value InstallDir "$PVSW_ROOT"
$PSREGEDIT -set -key "PS_HKEY_CONFIG/SOFTWARE/Pervasive Software/Products/PSQL Client/InstallInfo" -value InstallData "$PVSW_ROOT"
$PSREGEDIT -set -key "PS_HKEY_CONFIG/SOFTWARE/Pervasive Software/Products/PSQL Client/InstallInfo" -value ProductVersion $PRODUCT_VER
chown -Rf $USERNAME:$GROUPNAME $PS_REGISTRY
echo

echo "++++ Restoring the configuration from previous install..."
ETCDIR=$PVSW_ROOT/etc
SAVESUFFIX=rpmsave
SAVEPKGSUFFIX=pkgsave
# SAVEPKGSUFFIX is to save the file when tarball install/uninstall is done
if [ ! -f $ETCDIR/$cfgfile.$SAVESUFFIX ] ; then
    for cfgfile in odbc.ini ; do
        if [ -f $ETCDIR/$cfgfile.$SAVEPKGSUFFIX ] ; then
            if [ -f $ETCDIR/$cfgfile ] ; then
                echo "Archiving $cfgfile to $ETCDIR/$cfgfile.org"
                mv $ETCDIR/$cfgfile $ETCDIR/$cfgfile.org
            fi
            echo "Restoring previous $cfgfile.$SAVEPKGSUFFIX to $ETCDIR/$cfgfile"
            mv $ETCDIR/$cfgfile.$SAVEPKGSUFFIX $ETCDIR/$cfgfile
        fi
    done
else
    for cfgfile in odbc.ini ; do
        if [ -f $ETCDIR/$cfgfile.$SAVESUFFIX ] ; then
            if [ -f $ETCDIR/$cfgfile ] ; then
                echo "Archiving $cfgfile to $ETCDIR/$cfgfile.org"
                mv $ETCDIR/$cfgfile $ETCDIR/$cfgfile.org
            fi
            echo "Restoring previous $cfgfile.$SAVESUFFIX to $ETCDIR/$cfgfile"
            mv $ETCDIR/$cfgfile.$SAVESUFFIX $ETCDIR/$cfgfile
        fi
    done
fi
chmod a-wrx $ODBCINI
chmod a+r $ODBCINI
chmod ug+w $ODBCINI
rm -f $ETCDIR/psql
echo


# PCC, Builder and Docs configurations
#
# BEGIN CHECK FOR 32-bit gtk2 (Required for PCC)
#
GOT_GTK=1;

if [ -f $PVSW_ROOT/bin/pcc -o -f $PVSW_ROOT/bin/builder ] ; then
    if [ -f $PVSW_ROOT/bin/pcc ] ; then
        echo Initializing pcc...

        # If this is Darwin.
        if [ $(uname -s) = "Darwin" ]
        then
            $PVSW_ROOT/bin/PSQL\ Control\ Center.app/Contents/MacOS/pcc -clean -initialize
        else
            echo "$PVSW_ROOT/bin/pcc -clean -initialize" | $SU $USERNAME $SU_ARGS
        fi

        echo 
    fi
    if [ -f $PVSW_ROOT/bin/builder ] ; then
        echo Initializing builder...

        # If this is Darwin.
        if [ $(uname -s) = "Darwin" ]
        then
            $PVSW_ROOT/bin/PSQL\ DDF\ Builder.app/Contents/MacOS/builder -clean -initialize
        else
            echo "$PVSW_ROOT/bin/builder -clean -initialize" | $SU $USERNAME $SU_ARGS
        fi

        echo 
    fi
fi

if [ -d $PVSW_ROOT/bin/configuration ] ; then
    rm -Rf $PVSW_ROOT/bin/configuration/org.eclipse.help.base/index
    chmod 777 -Rf $PVSW_ROOT/bin/configuration 2>/dev/null
    chown -Rf $USERNAME:$GROUPNAME $PVSW_ROOT/bin/configuration
fi
if [ -d $PVSW_ROOT/docs ] ; then
    chmod -R 555 $PVSW_ROOT/docs
fi
PSQLHOME=`echo "echo ~" | $SU $USERNAME $SU_ARGS`
chown -Rf $USERNAME:$GROUPNAME $PSQLHOME

# If this is Darwin.
if [ $(uname -s) = "Darwin" ]
then
    APPLICATIONS="/Applications"

    # If there is an applications directory.
    if [ -d "$APPLICATIONS" ]
    then
        ACTIAN_PSQL_13="$APPLICATIONS/Actian PSQL 13"
        UTILITIES="$ACTIAN_PSQL_13/Utilities"
        echo "+++++ Populating the \"$ACTIAN_PSQL_13\" directory...."

        # If all of this succeeds.
        if (test -d "$ACTIAN_PSQL_13" || mkdir "$ACTIAN_PSQL_13") && \
            chmod 755 "$ACTIAN_PSQL_13" && \
            (test -d "$UTILITIES" || mkdir "$UTILITIES") && \
            chmod 755 "$UTILITIES" && \
            cp -r "$PVSW_ROOT/bin/scripts/PSQL Uninstall.app" "$ACTIAN_PSQL_13" && \
            chmod -R 755 "$ACTIAN_PSQL_13/PSQL Uninstall.app" && \
            cp "$PVSW_ROOT/bin/scripts/messages" "$ACTIAN_PSQL_13/.messages" && \
            chmod 644 "$ACTIAN_PSQL_13/.messages" && \
            cp "$PVSW_ROOT/bin/scripts/uninstall" "$ACTIAN_PSQL_13/.uninstall" && \
            chmod 755 "$ACTIAN_PSQL_13/.uninstall" && \
            cp -r "$PVSW_ROOT/bin/PSQL Control Center.app" "$ACTIAN_PSQL_13" && \
            chmod -R 755 "$ACTIAN_PSQL_13/PSQL Control Center.app" && \
            cp -r "$PVSW_ROOT/bin/PSQL DDF Builder.app" "$UTILITIES" && \
            chmod -R 755 "$UTILITIES/PSQL DDF Builder.app"
        then
            :
        else
            echo "Warning: Unable to correctly populate the \"$ACTIAN_PSQL_13\" directory."
        fi

        echo
    fi
fi

if [ -f $PVSW_ROOT/docs/readme_psql.htm ] ; then
echo View the README file located at:
echo "    $PVSW_ROOT/docs/readme_psql.htm"
echo
fi
if [ -f $PVSW_ROOT/bin/plugins/com.pervasive.psql.docs_1.0.0.jar ] ; then
echo From the PSQL Control Center, you may browse the complete PSQL documentation by running:
echo "    $PVSW_ROOT/bin/pcc"
echo
fi

echo Install has successfully completed.

echo
